package org.apache.bcel.verifier;

import java.util.Iterator;
import javax.swing.event.ListDataEvent;
import org.apache.commons.lang3.ArrayUtils;
import java.util.TreeSet;
import java.util.ArrayList;
import java.util.Set;
import javax.swing.event.ListDataListener;
import java.util.List;
import javax.swing.ListModel;

public class VerifierFactoryListModel implements VerifierFactoryObserver, ListModel<String>
{
    private final List<ListDataListener> listeners;
    private final Set<String> cache;
    
    public VerifierFactoryListModel() {
        this.listeners = new ArrayList<ListDataListener>();
        this.cache = new TreeSet<String>();
        VerifierFactory.attach(this);
        this.update(null);
    }
    
    @Override
    public synchronized void addListDataListener(final ListDataListener l) {
        this.listeners.add(l);
    }
    
    @Override
    public synchronized String getElementAt(final int index) {
        return this.cache.toArray(ArrayUtils.EMPTY_STRING_ARRAY)[index];
    }
    
    @Override
    public synchronized int getSize() {
        return this.cache.size();
    }
    
    @Override
    public synchronized void removeListDataListener(final ListDataListener l) {
        this.listeners.remove(l);
    }
    
    @Override
    public synchronized void update(final String s) {
        final Verifier[] verifiers = VerifierFactory.getVerifiers();
        final int verifierLen = verifiers.length;
        this.cache.clear();
        for (final Verifier verifier : verifiers) {
            this.cache.add(verifier.getClassName());
        }
        for (final ListDataListener listener : this.listeners) {
            listener.contentsChanged(new ListDataEvent(this, 0, 0, verifierLen - 1));
        }
    }
}
